import 'package:objd/core.dart';

/// The ClickEvent Module uses the trigger of a carrot on a stick to register right clicks with a scoreboard and execute a [onClick] Widget.
class CustomClickEvent extends Module {
  String path;
  String name;
  Widget onClick;
  Item selectedItem;
  String score;
  Score _score;

  /// The ClickEvent Module uses the trigger of a carrot on a stick to register right clicks with a scoreboard and execute a [onClick] Widget.
  ///
  /// This module has to be executed every tick to work!
  ///
  /// | constructor |  |
  /// |--|--|
  /// |name|used to identify different click Events|
  /// |onClick| Widget that is executed if the COAS is clicked |
  /// |selectedItem| just triggers the click if this Item is selected(optional) |
  /// |path| path to create the click handler function(default = `events/`) |

  CustomClickEvent(
    this.selectedItem,
  {
    this.name = "RC_Name",
    this.onClick,
    this.path = 'events/',
    this.score = "rc_name",
  }) {
    _score = Score.fromSelected(score);
  }

  @override
  Widget generate(Context context) {
    return For.of([
      Scoreboard(
        score,
        type: 'minecraft.custom:minecraft.talked_to_villager',
        addIntoLoad: true,
      ),

      Team.add("noColission"),
      Team.modify("noColission", collision: ModifyTeam.never),

      Tag(name + "Holding2", entity: Entity.All(tags: [name+"Holding"])),
      
      Entity.All(
          nbt: {'SelectedItem': selectedItem.getMap()},
      ).addTag(name + "Holding"),
      Entity.All(
          strNbt: '!{SelectedItem:{id:"' + selectedItem.type.toString() + '",tag:' + selectedItem.tag.toString() + '}}',
      ).removeTag(name + "Holding"),

      Execute.asat(
        Entity.All(tags:["!" + name + "Holding2", name+"Holding"]),
        children: [
          Command('/summon minecraft:armor_stand ^ ^ ^-0.5 {Invisible:1b,Invulnerable:1b,Marker:1b,NoGravity:1b,Passengers:[{id:"minecraft:villager",ActiveEffects:[{Id:14b,Amplifier:255b,Duration:19999980,ShowParticles:0b}],Invulnerable:1b,NoAI:1b,NoGravity:1b,Silent:1b,Tags:[' + name + ', ' + name + 'Vill]}],Silent:1b,Tags:[' + name +', ' + name + 'Stand]}'),
          Team.join("noColission", Entity(tags: [name + "Vill"],),),
        ],
      ),
      Execute.asat(
        Entity.All(tags:[name + "Holding2", "!" + name + "Holding"]),
        children: [
          Command('/tp @e[tag=' + name + ',distance=..3,sort=nearest,limit=2] ~ -200 ~'),
          Command('/kill @e[tag=' + name + ',distance=..3,sort=nearest,limit=2]'),
        ],
      ),

      Entity.All(tags: [name+"Holding2"]).removeTag(name+"Holding2"),
      
      Execute(
        as: Entity(tags: [name]),
        at: Entity.Self(),
        If: Condition.not(Condition.entity(Entity.Player(distance: Range(to: 3)))),
        location: Location.rel(y: -1),
        children: [
          Command('/tp @s ~ -500 ~'),
          Command('/kill @s'),
        ],
      ),
      
      Execute(
        as: Entity(tags: [name + "Holding"]),
        at: Entity.Self(),
        location: Location.rel(y: 1.5),
        children: [
          Command('/tp @e[tag=' + name + 'Stand,distance=..3,sort=nearest,limit=1] ^ ^ ^'),
        ],
      ),


      Execute.asat(
          Entity.All(
              scores: [
                _score.matchesRange(
                  Range(from: 1),
                ),
              ],
              nbt: selectedItem != null
                  ? {'SelectedItem': selectedItem.getMap()}
                  : null),
          children: [File.execute(path + score, create: false)]),
    ]);
  }

  @override
  List<File> registerFiles() => <File>[
        File(
          path + score,
          child: For.of([
            onClick,
            _score.reset(),
          ]),
        ),
      ];
}
